`area.between.curves` <-
function(x, f1, f2, xrange=c(0,1))
{
	a<-0.0;
	for(i in 1:length(x)) {
		if(x[i]>=xrange[1] & x[i]<=xrange[2]) {
			if(i==1) {
				lhs<-0  
			} else if(x[i-1]<xrange[1]) {
					lhs<-xrange[1]
				} else lhs<-x[i-1];
			if(i==length(x)) { 
				rhs<-x[i]
			} else if(x[i+1]>xrange[2]) {
				rhs<-xrange[2]; 
			} else rhs<-x[i+1];
			a<-a+(f2[i]-f1[i])*(rhs-lhs)/2;	
		} else if(i!=1) if(x[i-1]>=xrange[1] & x[i-1]<=xrange[2]) {
			y1<-f1[i-1]+(f1[i]-f1[i-1])*(xrange[2]-x[i-1])/(x[i]-x[i-1])
			y2<-f2[i-1]+(f2[i]-f2[i-1])*(xrange[2]-x[i-1])/(x[i]-x[i-1])
			a<-a+(y2-y1)*(xrange[2]-x[i-1])/2;
		} else if(i!=length(x)) if(x[i+1]>=xrange[1] & x[i+1]<=xrange[2]) {
			y1<-f1[i]+(f1[i+1]-f1[i])*(xrange[1]-x[i])/(x[i+1]-x[i])
			y2<-f2[i]+(f2[i+1]-f2[i])*(xrange[1]-x[i])/(x[i+1]-x[i])

			a<-a+(y2-y1)*(x[i+1]-xrange[1])/2;
		}
	}
	return(a)
}


######################################################################################
##STOCHASTIC SIMULATION OF A TIME-HOMOGENOUS BIRTH-DEATH PROCESS
##PARAMETERS: N0=starting number of lineages, ks=speciation rate, ep=relative rate of extinction, finaltime=endpoint for simulation
######################################################################################
BDsim <- function(nStart, b, d, times) {
	n <- nStart
	t <- 0
	pop<-numeric(length(times))
	pop[]=0
	pop[1] <- nStart
	
	times<-c(0, sort(times))
	i=2
	while(1) {
		if(t>times[i]) {
			m=max(which(t>times))
			pop[i:m]=n;
			i=m+1;	
		}

		waittime <- rexp(1, rate=n*(b+d))
		t <- t+ waittime
		if(t > max(times)) {
			break 
		} else {
			ran <- runif(1)
			if(ran<=b/(b+d)) n <- n+1						else n <- n-1
		}
		if(n==0) break
	}
	pop[i:length(times)]=n
	res=cbind(times, pop)
	colnames(res)=c("Time", "n")		
	return(res)
}	



`birthdeath.tree` <-
function (b, d, time.stop=0, taxa.stop=0, seed=0, print.seed=FALSE, return.all.extinct=TRUE){

# December 6 2005 Jason T. Weir
# Modified by Luke J. Harmon
# The following simulates Yule trees to a given time T

if(seed==0) seed=set.seed.clock(print=print.seed);

if(time.stop==0 & taxa.stop==0)
	stop("Must have stopping criterion\n");
	
while(1) {

edge <- rbind(c(1, 2), c(1, 3)) # this is a starting edge matrix
edge.length <- rep(NA, 2)
stem.depth <- numeric(2)
alive<-rep(TRUE, 2) # marker for live lineages
t <- 0 #time at any point in the tree
next.node<-4



############
repeat{
	
	
	if(taxa.stop) if(sum(alive)>=taxa.stop) break;
	if(sum(alive)==0) break;
	dt<-rexp(1, sum(alive)*(b+d));
	  t<-t+dt;
	  if(time.stop) if(t>=time.stop) {
		t<-time.stop;
		break;
	  }
	  r<-runif(1)
         if(r<=b/(b+d)) {###4 #this creates a bifucation in the tree
	          	random_lineage <- round(runif(1, min=1, max=sum(alive)))
			e<-matrix(edge[alive,], ncol=2)
			parent<-e[random_lineage,2]
			alive[alive][random_lineage]<-FALSE
			edge<-rbind(edge, c(parent, next.node), c(parent, next.node+1))
			next.node<-next.node+2
			alive<-c(alive, TRUE, TRUE)
			stem.depth<-c(stem.depth, t, t)
			x<-which(edge[,2]==parent)
			edge.length[x]<-t-stem.depth[x]
			edge.length<-c(edge.length, NA, NA)
            }###4

         else {###4 This terminates one of the current lineages on the tree
                random_lineage <- round(runif(1, min=1, max=sum(alive)))
		    edge.length[alive][random_lineage]<-t-stem.depth[alive][random_lineage];
          	    alive[alive][random_lineage]<-FALSE
            }###4
      }#1A
      
if(return.all.extinct==T | sum(alive)>1) break;
}
edge.length[alive]<-t-stem.depth[alive]
n<--1;
for(i in 1:max(edge)) {
	if(any(edge[,1]==i)) {
		edge[which(edge[,1]==i), 1]<-n
		edge[which(edge[,2]==i), 2]<-n
		n<-n-1
		}
	}

	
	edge[edge>0]<-1:sum(edge>0)

	tip.label<-1:sum(edge>0)
    	mode(edge) <- "character"
    	mode(tip.label) <- "character"
    	obj <- list(edge = edge, edge.length = edge.length, tip.label=tip.label)
    	class(obj) <- "phylo"
    	obj<-old2new.phylo(obj)
    	obj<-read.tree(text=write.tree(obj))	
    obj
}

`set.seed.clock` <-
function(print=F){
	date = date()
 	seed1 = as.numeric(strsplit(substring(date,12,19),":")[[1]])%*%c(1,100,10000)
 	seed <- runif(1, min=0, max=50) * seed1
 	set.seed(seed)
 	if(print) cat("Seed = ", seed, "\n");
 	seed[1,1]
}




############################################################
###############TREE TRANSFORMATIONS#########################
############################################################
## a few functions modified 11 dec 07 to accommodate ultrametric trees; see comments below

###LAMBDA###
## modified 11 dec 07
lambdaTree <- function(phy, lambda)
{
	originalDiag = diag(vcv.phylo(phy)) ## added to fix the final edge length problem in non-ultrametric tree
	original.rtt <- max(diag(vcv.phylo(phy))) ## generalizes to a non-ultrametric tree
	  ## following line assumes ultrametricity
	  ## original.rtt <- max(branching.times(phy))
	ltree <- phy
	ltree.old <- new2old.phylo(phy) #needed due to change in ape tree format
	ltree$edge.length <- ltree$edge.length * lambda #shortens internal branches in proportion to lambda
	  ## The following assignment of terminal branch fails when the tree is non-ultrametric
	  ## t <- original.rtt * (1-lambda) #needed to rescale tree back to its original root-to-tip length
	which(ltree.old$edge[,2] > 0) -> terminal.edges #identifies termal edges to be extended for recovery of original rtt length; based on old tree format in which nodes are coded as negative values
	terminalEdgesOrdered <- match(match(labels(originalDiag), ltree$tip.label, nomatch = F), ltree$edge[,2])
	ltree$edge.length[terminalEdgesOrdered] <- 
	ltree$edge.length[terminalEdgesOrdered] + (originalDiag - (originalDiag * lambda)) ## generalizes to a non-ultrametric tree
	  ## The following didn't work on a non-ultrametric tree
	  ## ltree$edge.length[terminal.edges] + t -> ltree$edge.length[terminal.edges] -- extends terminal edges
	estMinLambda <- 
	  ((min(phy$edge.length[terminalEdgesOrdered]) 
	  + originalDiag[phy$edge[,2][match(min(phy$edge.length[terminalEdgesOrdered]), phy$edge.length)]]) 
	  / originalDiag[phy$edge[,2][match(min(phy$edge.length[terminalEdgesOrdered]), phy$edge.length)]] )
	if(any(ltree$edge.length < 0)) message("Lambda values too large imply negative terminal branch lengths. \nLambda should not exceed an estimated ", round(estMinLambda, 3), " on this tree.")
	return(ltree)
}

###DELTA###
## checked 11 dec 07; works fine
## Modified Jan 2. 08 by Steve Kembel to work when trees have internal branch labels

deltaTree <- function (phy, delta, rescale = T) 
{
    tmp <- as.numeric(phy$edge)
    times <- branching.times(phy)
    original.rtt.length <- max(times)
    times = max(times) - times
    max(times)
    tips <- length(phy$tip.label)
    res <- phy
    for (i in 1:length(phy$edge.length)) {
        bl <- phy$edge.length[i]
        age = times[phy$edge[i, 1] - tips]
        res$edge.length[i] <- (age + bl)^delta - age^delta
    }
    if (rescale == T) 
        res <- rescaleTree(res, original.rtt.length)
    res
}

###TWORATE#####

tworateTree<-function(phy, breakPoint, endRate) 
{
	times<-branching.times(phy)	
	for(i in 1:length(phy$edge.length)) {
		bl<-phy$edge.length[i]
		age=times[which(names(times)==phy$edge[i,1])] #gets tip to node length
		if((age-bl)<breakPoint) #identifies branches that are on the tip side of the break-point (i.e., young)
			phy$edge.length[i]<-(age-min(age, breakPoint))*1+(min(age, breakPoint)-(age-bl))*endRate #If an edge is entirely to the right of the breakpoint it is simply multipled by f.  However, if the edge extends across the breakpoint, we want to leave the part to the left of the BP unchanged (this is the first part of this equation, which multiplies this part by 1).  We then want to multiply the part of the edge that is to the right of the BP by f (this is the second part of this line).
		}
	phy
}

###LINEAR CHANGE###
linearchangeTree<-function(phy, endRate=NULL, slope=NULL)
{
	
    if(is.null(slope)&&is.null(endRate))
    	stop("Must supply either endRate or slope")
    	
	times<-branching.times(phy)
	names(times)<-(as.numeric(names(times)))
	rootdepth=max(times)
	
	
	
	if(is.null(slope)) {
		slope=(endRate-1)/rootdepth
		}
	

	getrate<-function(begin, end)
	{
		br<-1+begin*slope
		er<-1+end*slope
		if(br>0 & er>0) return((br+er)/2)
		if(br<0 & er<0) return(0)
		ii<- -1/slope
		return(br*(ii-begin)/(2*(end-begin)))
	}

	for(i in 1:length(phy$edge.length)) {
		bl<-phy$edge.length[i]
		age=rootdepth-times[which(names(times)==phy$edge[i,1])]
		end=age+bl
	
		phy$edge.length[i]<-phy$edge.length[i]*getrate(age, end)

		

	}
	phy	
}

###RESCALING###
## modified 11 dec 07
rescaleTree<-function(phy, totalDepth)
{
	## following line added to accommodate ultrametric trees
	d <- max(diag(vcv.phylo(phy)))
	  ## assumes ultrametricity
	  ## d<-max(branching.times(phy))
	phy$edge.length<-(phy$edge.length/d)*totalDepth
	phy
}

exponentialchangeTree<-function (phy, endRate=NULL, a=NULL) 
{
	

    if(is.null(a)&&is.null(endRate))
    	stop("Must supply either endRate or a")
    	
   	times <- branching.times(phy)
    d<-max(times)
    if(is.null(a))
 	   a<-log(endRate)/d
 	if(a==0) return(phy)   
    names(times) <- (as.numeric(names(times)))
    for (i in 1:length(phy$edge.length)) {
        bl <- phy$edge.length[i]
        age = times[which(names(times) == phy$edge[i, 1])]
        t1 = max(times) - age
        t2 = t1+bl
        phy$edge.length[i] = (exp(a*t2)-exp(a*t1))/(a)
    }
    phy
}

speciationalTree<-function(phy)
{
	phy$edge.length[]=1
	phy	
}

ouTree<-function(phy, alpha) {
	times <- branching.times(phy)
    names(times) <- (as.numeric(names(times)))
    Tmax<-times[1]
    phy2<-phy
    for (i in 1:length(phy$edge.length)) {
        bl <- phy$edge.length[i]
        age = times[which(names(times) == phy$edge[i, 1])]
        t1 = max(times) - age
        t2 = t1+bl
        phy2$edge.length[i] = (1/(2*alpha))*exp(-2*alpha * (Tmax-t2)) * (1 - exp(-2 * alpha * t2)) - 
        						(1/(2*alpha))*exp(-2*alpha * (Tmax-t1)) * (1 - exp(-2 * alpha * t1))
    }
    phy2
	
}

kappaTree<-function(phy, kappa)
{
	phy$edge.length<-phy$edge.length^kappa
	return(phy)	
}
`disp.calc` <-
function(data, disp="avg.sq")
{
	if(disp=="avg.sq") {
		d<-dist(data, method="euclidean")^2
		r<-mean(d)
	}
	else if(disp=="avg.manhattan") {
		d<-dist(data, method="manhattan")
		r<-mean(d)
	}
	else if(disp=="num.states") {
		f<-function(x) length(unique(x))
		d<-apply(data, 2, f)
		r<-mean(d)
	}
	else r<-0;
	return(r)
}



`dtt` <-
function(phy, data, data.names=NULL, disp="avg.sq")
{
	phy$node.label<-NULL
	td<-treedata(phy, data, data.names)
	phy2<-td$phy
	phy<-new2old.phylo(td$phy)
	
	result<-numeric()
	

	node.depth<-branching.times(phy2);
	stem.depth<-numeric();
	stem.depth[1]<-node.depth[1];
	for(i in 2:phy2$Nnode) {
		anc<-which(as.numeric(phy$edge[,2])==-i)
		stem.depth[i]<-node.depth[names(node.depth)==phy2$edge[anc,1]]
	}
		
	ltt<-sort(node.depth, decreasing=TRUE)
	node.depth<-node.depth/max(ltt);
	stem.depth<-stem.depth/max(ltt);
	ltt<-ltt/max(ltt);
	if(length(dim(td$data))==2) {
		d<-tip.disparity(phy2, td$data, disp=disp);
		result[1]<-d[1]
		for(i in 2:length(ltt)) {
			x<-d[stem.depth>=ltt[i-1]&node.depth<ltt[i-1]]
			if(length(x)==0) result[i]=0
			else result[i]<-mean(x);
		}
		result[length(ltt)+1]<-0;
		if(result[1]>0)
			result<-result/result[1];
			
	} else {
		if(length(dim(td$data))!=3)
			stop("Error in data");
		
		for(i in 1:dim(td$data)[3]) {
			pp<-as.matrix(td$data[,,i])
			d<-tip.disparity(phy2, pp, disp=disp);
			y<-numeric()
	
			y[1]<-d[1]
			for(j in 2:length(ltt)) {
				x<-d[stem.depth>=ltt[j-1]&node.depth<ltt[j-1]]
				if(length(x)==0) y[j]=0
				else y[j]<-mean(x);
			}
			y[length(ltt)+1]<-0;
			if(y[1]>0)
			y<-y/y[1];
			
			result<-cbind(result, y)
		}
	}
	
	return(result);	
}

dtt.full<-function(phy, data, data.names=NULL, disp="avg.sq", nsims=1000, mdi.range=c(0,1))
{
	td<-treedata(phy, data, data.names)

	dtt.data<-dtt(td$phy, td$data, disp=disp)
	ltt<-sort(branching.times(td$phy), decr=TRUE)
	ltt<-c(0, (max(ltt)-ltt)/max(ltt));
	plot(ltt, dtt.data, type="l", lwd=2, xlab="Relative time", ylab="Disparity");
	
	s<-ic.sigma(td$phy, td$data)
	sims<-sim.char(td$phy, s, nsims)

	dtt.sims<-dtt(td$phy, sims)
	mean.sims<-apply(dtt.sims, 1, mean)

	lines(ltt, mean.sims, lty=2)


	MDI<-area.between.curves(ltt, apply(dtt.sims, 1, median), dtt.data, mdi.range)


	return(list(dtt.data=dtt.data, dtt.sims=dtt.sims, times=ltt, MDI=MDI))
	
}


`fitContinuous` <-
function(phy, data, data.names=NULL, model=c("BM", "OU", "lambda", "kappa", "delta", "EB", "white", "trend"), bounds=NULL,  meserr=NULL)
{
	
	# sort is T because sub-functions assume data are in
	# this particular order
	
	model<-match.arg(model)
	
	td<-treedata(phy, data, data.names, sort=T)

	ntax=length(td$phy$tip.label)

	if(is.null(meserr)) {
		me=td$data
		me[]=0
		meserr=me	
	} else if(length(meserr)==1) {
		me=td$data
		me[]=meserr
		meserr=me
	} else if(is.vector(meserr)) {
		if(!is.null(names(meserr))) {
			o<-match(rownames(td$data), names(meserr))
			if(length(o)!=ntax) stop("meserr is missing some taxa from the tree")
			meserr<-as.matrix(meserr[o,])
		} else {
			if(length(meserr)!=ntax) stop("No taxon names in meserr, and the number of taxa does not match the tree")
			me<-td$data
			me[]=meserr
			meserr=me
		}
	} else {
		if(!is.null(rownames(meserr))) {
			o<-match(rownames(td$data), rownames(meserr))
			meserr=meserr[o,]
		} else {
			if(sum(dim(meserr)!=dim(td$data))!=0)
				stop("No taxon names in meserr, and the number of taxa does not match the tree")
			print("No names in meserr; assuming that taxa are in the same order as tree")	
		}
	}

	#--------------------------------
    #---    PREPARE DATA LIST     ---
    #--------------------------------
	ds			<- list()
   		ds$tree 		<- td$phy          # TIP data 
    #--------------------------------
    #--- SET MODEL SPECIFICATIONS ---
    #--------------------------------
    cat("Fitting ", model, "model:\n")
    #-----------------------------
    #---  SET PARAMETER BOUNDS ---
    #-----------------------------
    #---- DEFAULT BOUNDS
    bounds.default			 <- matrix(c(0.00000001, 20, 0.0000001,1, 0.000001, 1, 0.00001, 2.999999, 0.0000001, 50, -3, 0, 0.0000000001, 100, -100, 100), nrow=8, ncol=2, byrow=TRUE)
    rownames(bounds.default) <- c("beta", "lambda", "kappa", "delta", "alpha", "a", "nv", "mu");
    colnames(bounds.default) <- c("min", "max")

 	#---- USER DEFINED PARAMETER BOUNDS
 	if (is.null(bounds)) {
 		bounds <- bounds.default       # USE DEFAULTS
 	}else{
 		if (class(bounds)!="list"){
 			stop("Please specify user defined parameter bounds as a list()")
 		}else{
 			specified   <- !c(is.null(bounds$beta), is.null(bounds$lambda), 
 							  is.null(bounds$kappa), is.null(bounds$delta),  is.null(bounds$alpha), is.null(bounds$a),
 							  is.null(bounds$nv), is.null(bounds$mu)
 							  )
 			bounds.user <- matrix(c(bounds$beta, bounds$lambda, bounds$kappa, bounds$delta, bounds$alpha, bounds$a, bounds$nv, bounds$mu), 
 								  nrow=sum(specified), ncol=2, byrow=TRUE
 								  )
 			rownames(bounds.user) <- c("beta", "lambda", "kappa", "delta", "alpha", "a",  "nv", "mu")[specified]
   	 		colnames(bounds.user) <- c("min", "max")
  
   	 		#----  SET FINAL SEARCH BOUNDS
 			bounds <- bounds.default
 			bounds[specified,] <- bounds.user     # Final Bounds
   		} # END if list
   	}  # END user bound if loop
   	#--------------------------------
    #---   APPEND MODEL SETTINGS  ---
    #--------------------------------
  	ds$bounds <- data.frame(t(bounds))

  	ds$model  <- model
  	#--------------------------------
    #---        FIT MODEL         ---
    #--------------------------------
    result<-list()
    for(i in 1:ncol(td$data)) {
    	ds$data=td$data[,i]
    	ds$meserr=meserr[,i]
  		result[[i]]<-fitContinuousModel(ds, print=print)
  		if(!is.null(colnames(td$data))) names(result)[i]<-colnames(td$data)[i] else names(result)[i]<-paste("Trait", i, sep="")

  	}
  	result
}


`fitContinuousModel` <-
function(ds, print=TRUE)
{
	bounds 	<- ds$bounds
	model 	<- ds$model
	n 		<- length(ds$data)

	#----- MINIMIZE NEGATIVE LOG LIKELIHOOD
	
	beta.start<-var(ds$data)/max(branching.times(ds$tree))


	out         <- NULL
	
	y			<- ds$data				# TIP data
	tree		<- ds$tree			# Tree
	meserr		<- ds$meserr
	n			<- length(y)
	

	#----------------------------------
	#-----       DEFAULT FIT      -----
	#----------------------------------
	if (model=="BM") {
		k<-2
	
		vcv<-vcv.phylo(tree)

		start=log(beta.start)
		lower=log(bounds[1,"beta"])
		upper=log(bounds[2,"beta"])
		
		foo<-function(x) {
			vv<-exp(x)*vcv
			diag(vv)<-diag(vv)+meserr^2
			mu<-phylogMean(vv, y)
			mu<-rep(mu, n)
			-dmvnorm(y, mu, vv, log=T)
		}
		
		o<-optim(foo, p=start, lower=lower, upper=upper, method="L")
			
		results<-list(lnl=-o$value, beta= exp(o$par))

	#----------------------------------
	#-----       LAMBDA ONLY      -----
	#----------------------------------
	} else if (model=="lambda"){
		k<-3
		
		start=log(c(beta.start, 0.5))
		lower=log(bounds[1,c("beta","lambda")])
		upper=log(bounds[2,c("beta","lambda")])
		
		
		foo<-function(x) {


			vcv<-vcv.phylo(tree)

			index			<-	matrix(TRUE, n,n)
			diag(index)		<- FALSE
			vcv[index] 	<- vcv[index]*exp(x[2])
			
			vv<-exp(x[1])*vcv

			
			diag(vv)<-diag(vv)+meserr^2
			mu<-phylogMean(vv, y)
			mu<-rep(mu, n)
			
			-dmvnorm(y, mu, vv, log=T)
		}
		o<-optim(foo, p=start, lower=lower, upper=upper, method="L")
			
		results<-list(lnl=-o$value, beta= exp(o$par[1]), lambda=exp(o$par[2]))

	#----------------------------------
	#-----        KAPPA ONLY      -----
	#----------------------------------
	} else if (model=="kappa"){
		k<-3
		start=log(c(beta.start, 0.5))
		lower=log(bounds[1,c("beta","kappa")])
		upper=log(bounds[2,c("beta","kappa")])
				
		
		foo<-function(x) {

			t<-kappaTree(tree, kappa=exp(x[2]))
			vcv<-vcv.phylo(t)

			
			vv<-exp(x[1])*vcv

			
			diag(vv)<-diag(vv)+meserr^2
			mu<-phylogMean(vv, y)
			mu<-rep(mu, n)
			
			-dmvnorm(y, mu, vv, log=T)
		}
		o<-optim(foo, p=start, lower=lower, upper=upper, method="L")
		
		results<-list(lnl=-o$value, beta= exp(o$par[1]), lambda=exp(o$par[2]))


	#----------------------------------
	#-----        DELTA ONLY      -----
	#----------------------------------	
	} else if (model=="delta"){
		
		k<-3
		start=log(c(beta.start, 0.5))
		lower=log(bounds[1,c("beta","delta")])
		upper=log(bounds[2,c("beta","delta")])
		
		foo<-function(x) {

			t<-deltaTree(tree, delta=exp(x[2]))
			vcv<-vcv.phylo(t)

			
			vv<-exp(x[1])*vcv

			
			diag(vv)<-diag(vv)+meserr^2
			mu<-phylogMean(vv, y)
			mu<-rep(mu, n)
			
			-dmvnorm(y, mu, vv, log=T)
		}
		o<-optim(foo, p=start, lower=lower, upper=upper, method="L")
		
		results<-list(lnl=-o$value, beta= exp(o$par[1]), delta=exp(o$par[2]))	
	#----------------------------------
	#-----        WHITE NOISE     -----
	#----------------------------------	
	} else if (model=="white"){
		
		k<-2
		start=c(mean(y), log(var(y)))
		lower=c(-Inf, log(bounds[1,"nv"]))
		upper=c(Inf, log(bounds[2, "nv"]))
		
		lnl.noise<- function (p, x, se)
		# p is the vector of parameters, tree is not needed
		# x and se are trait means and std errors
		{
  			## prep parameters
  			root<- p[1]	# trait value of root of tree (also optimum)
  			vs<- exp(p[2])		# white noise variance 
  			n<- length(x)
  			VV<- diag(vs, nrow=n)	
  			diag(VV)<- diag(VV) + se^2	# add sampling variance to diagonal
    
  			## logl
  			M<- rep(root,n)
  			-dmvnorm(x, M, VV, log=TRUE)
		}
		
		o<- optim(start, fn=lnl.noise, x=y, se=meserr, lower=lower, upper=upper, method="L")
		
		results<-list(lnl=-o$value, mean= o$par[1], nv=exp(o$par[2]))	
	#----------------------------------
	#-----        TREND           -----
	#----------------------------------	
	} else if (model=="trend"){
		
		k<-3
		vcv<- vcv.phylo(tree)
  		ww<- lm(y ~ diag(vcv))
  		p0<- c(phylogMean(vcv, y), var(y)/max(branching.times(tree)), coef(ww)[2])
		if(is.na(p0[3])) {
			p0[3]<-0
			if(is.ultrametric(tree))
				cat("WARNING: Cannot estimate a trend with an ultrametric tree; lnl will be the same as the BM model")
		}
		lower=c(-Inf, log(bounds[1,"beta"]), bounds[1,"mu"])
		upper=c(Inf, log(bounds[2,"beta"]), bounds[2,"mu"])
		

		lnl.BMtrend<- function(p, vcv, x, se)
		# p is vector of parameters, tr is tree
		# x and se are vectors of trait means and standard errors
		{
  			## prep parameters
  			root<- p[1]	# trait value of root of tree
  			vs<- exp(p[2])		# BM variance 
  			ms<- p[3]		# BM trend
  			VV<- vs*vcv	
  			diag(VV)<- diag(VV) + se^2	# add sampling variance to diagonal
  			n<- length(x)
  
 			## logl
  			M<- root+ ms*diag(vcv)
  			- dmvnorm(x, M, VV, log=TRUE)
  		}

		o<- optim(p0, fn=lnl.BMtrend, vcv=vcv, x=y, se=meserr, lower=lower, upper=upper, method="L")
		names(o$par)<-NULL
		results<-list(lnl=-o$value, mean= o$par[1], beta=exp(o$par[2]), mu=o$par[3])		
	#----------------------------------
	#-----        ALPHA ONLY      -----
	#----------------------------------			
	} else if (model=="OU"){
	## modified 12 dec 07 to call ouMatrix(x) instead of vcv.phylo(ouTree(x))

		k<-3
		
		start=log(c(beta.start, 0.5))
		lower=log(bounds[1,c("beta","alpha")])
		upper=log(bounds[2,c("beta","alpha")])
	
		vcvOrig<-vcv.phylo(tree)
		foo<-function(x) {
			vcv <- ouMatrix(vcvOrig, exp(x[2]))
			
			## t<-ouTree(tree, exp(x[2]))
			##vcv<-vcv.phylo(t)
			
			vv<-exp(x[1])*vcv
			diag(vv)<-diag(vv)+meserr^2
			
			mu<-phylogMean(vv, y)
			mu<-rep(mu, n)
			
			-dmvnorm(y, mu, vv, log=T)
		}
		
		outTries<-list()
		
		# First one: try near BM solution
		start=c(log(beta.start), -50)
		outTries[[1]]<-optim(foo, p=start, lower=lower, upper=upper, method="L")
		
		# Second one: try one with very strong constraints
		tv<-var(y)
		start=log(c(tv*2000, 1000))
		outTries[[2]]<-optim(foo, p=start, lower=lower, upper=upper, method="L")
	

	
		# Try ten random ones
		for(i in 1:10){
			while(1) {

				lower=c(runif(2, min=-20, max=-1))
				upper=lower+runif(2, min=0, max=10)
				start=c(runif(1, min=lower[1], max=upper[1]), runif(1, min=lower[2], max=upper[2]))
				te<-try(outTries[[i+2]]<-optim(foo, p=start, lower=lower, upper=upper, method="L"), silent=T)
				if(class(te)!="try-error") break
				}
				
		}
		
		# Try range of alphas
		atry<- -5:4
		stry<- log(tv*2*exp(atry))
		for(i in 1:10){
			while(1) {

				lower=c(-20, -20)
				upper=c(10, 10)
				start=c(stry[i], atry[i])
				te<-try(outTries[[i+12]]<-optim(foo, p=start, lower=lower, upper=upper, method="L"), silent=T)
				if(class(te)!="try-error") break
				}
				
		}
		
		
		
		ntries<-22
		ltry<-numeric(ntries)
		lsol<-matrix(nrow= ntries, ncol=2)
		for(j in 1:ntries) {
				ltry[j]<-outTries[[j]]$value
				lsol[j,]<-exp(outTries[[j]]$par)
			}

		ltd<-ltry-min(ltry)
		b<-min(which(ltry==min(ltry)))

		gc<-which(ltd<0.01)
		us<-lsol[gc,1]
		usc<-sum((us-min(us))>0.01)			
		out<-outTries[[b[1]]]	
		if(usc>1) {out$message="Warning: likelihood surface is flat."}
			
		if(out$convergence!=0) {out$message="Warning: may not have converged to a proper solution."}

		results<-list(lnl=-out$value, beta= exp(out$par[1]), alpha=exp(out$par[2]), convergence=out$convergence, message=out$message, k=k)


	#----------------------------------
	#-----        EB ONLY      -----
	#----------------------------------	
	} else if(model=="EB"){

		k<-3
		start=c(log(beta.start), 0.01)
		lower=c(log(bounds[1,"beta"]),bounds[1,"a"])
		upper=c(log(bounds[2,"beta"]),bounds[2,"a"])
		
		foo<-function(x) {
			t<-exponentialchangeTree(tree, a=x[2])

			vcv<-vcv.phylo(t)
			
			vv<-exp(x[1])*vcv
			diag(vv)<-diag(vv)+meserr^2
			
			mu<-phylogMean(vv, y)
			mu<-rep(mu, n)
			
			-dmvnorm(y, mu, vv, log=T)
		}
		o<-optim(foo, p=start, lower=lower, upper=upper, method="L")
			
		results<-list(lnl=-o$value, beta= exp(o$par[1]), a=o$par[2])	
	}
	
	results$aic<-2*k-2*results$lnl
	results$aicc<-2*k*(n-1)/(n-k-2)-2*results$lnl
	results$k<-k
	return(results) 

}



phylogMean<-function(phyvcv, data) 
{
	o<-rep(1, length(data))
	ci<-solve(phyvcv)
	
	m1<-solve(t(o) %*% ci %*% o)
	m2<-t(o) %*% ci %*% data
	
	return(m1 %*% m2)
	
	}
	
ouMatrix <- function(vcvMatrix, alpha) 
{
## follows Hansen 1997; does not assume ultrametricity (AH 12 dec 07)
## vectorized by LJH
  vcvDiag<-diag(vcvMatrix)
  diagi<-matrix(vcvDiag, nrow=length(vcvDiag), ncol=length(vcvDiag))
  diagj<-matrix(vcvDiag, nrow=length(vcvDiag), ncol=length(vcvDiag), byrow=T)

  Tij = diagi + diagj - (2 * vcvMatrix)
    
  vcvRescaled = (1 / (2 * alpha)) * exp(-alpha * Tij) * (1 - exp(-2 * alpha * vcvMatrix))
  return(vcvRescaled) 
}
    	
`fitDiscrete` <-
function(phy, data, model=c("ER", "SYM", "ARD"), treeTransform=c("none", "lambda", "kappa", "delta", "linearChange", "exponentialChange", "twoRate"), data.names=NULL, plotlnl=F, qLimits=c(0.0001, 1000), pLimits=c(0.00001, 10))
{
	
	model<-match.arg(model)

	

	
	treeTransform<-match.arg(treeTransform)
	if(treeTransform=="twoRate" & plotlnl==T) {
				cat("Plotting surfaces not supported for twoRate tree transformation\n")
				plotlnl=F
	}
	
	if(!is.ultrametric(phy)) {
		cat("Warning: some tree transformations in GEIGER might not be sensible for nonultrametric trees.\n")
		}

	if(hasZeroLengthTips(phy)) {
		cat("Warning: your tree has some zero-length tip branches. If the desendent species have different trait values, the likelihood will be zero and this approach will not work.\n")
		}

	td<-treedata(phy, data, data.names, sort=T)

	

	res<-list()

	for(i in 1:ncol(td$data))
	{

	
		if(treeTransform=="none") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x), model)
			}
			nep=0; pLow=-10; pHigh=log(1); pStart=NULL;		
		}	
		if(treeTransform=="lambda") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x[-1]), lambda=exp(x[1]), model=model)
			}	
			nep=1; pLow=-10; pHigh=log(1); pStart=0.1;
		}
		if(treeTransform=="delta") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x[-1]), delta=exp(x[1]), model=model)
				}
			nep=1; pLow=-10; pHigh=log(10);pStart=0.1;
		}
		if(treeTransform=="kappa") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x[-1]), kappa=exp(x[1]), model=model)
				}
			nep=1; pLow=-10; pHigh=log(1);pStart=0.1;
		}
		if(treeTransform=="linearChange") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x[-1]), endRate=exp(x[1]), linear=T, model=model)
				}
			nep=1; pLow=-10; pHigh=log(10);pStart=0.1;
		}
		if(treeTransform=="exponentialChange") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x[-1]), endRate=exp(x[1]), model=model)
				}
			nep=1; pLow=-10; pHigh=log(10);pStart=0.1;
		}
		if(treeTransform=="twoRate") {
			f<-function(x) {
				likelihoodDiscrete(td$phy, td$data[,i], exp(x[-(1:2)]), breakPoint=x[1], endRate=exp(x[2]), model=model)
				}
			mv<-max(branching.times(td$phy))	
			nep=2; pLow=c(mv/1000, -10); pHigh=c(mv, 10);pStart=c(0.1, 0.1);
		}
				
						
			nRateCats<-getRateCats(td$data[,i], model)
			
			outTries<-list()
			totalbl<-sum(td$phy$edge.length)
			minQ=log(0.01/totalbl)
			maxQ=log(1000/totalbl)
			ntries<-20
			ltry<-numeric(ntries)
			ltry[]<-NA
			lsol<-matrix(nrow= ntries, ncol=nRateCats+nep)
			sp<-numeric(nRateCats)
			qTries<-exp(-7:2)
			
			if(nep==0) {
				lower=rep(minQ, nRateCats)
				upper=rep(maxQ, nRateCats)
			} else {
				lower=c(pLow, rep(minQ, nRateCats))
				upper=c(pHigh, rep(maxQ, nRateCats))
			}
			
			cat("Finding the maximum likelihood solution\n")
			cat("[0        50      100]\n")
			cat("[")
			
			for(j in 1:10) {
				sp<-c(pStart, log(rep(qTries[j], nRateCats)))
				te<-try(outTries[[j]]<-optim(f, par=sp, method="L",  lower=lower, upper=upper), silent=T)
				
				if(class(te)!="try-error") {
					ltry[j]<-outTries[[j]]$value
					lsol[j,]<-exp(outTries[[j]]$par)
				}
				cat(".")

			}
			for(j in 1:10) {
				sp<-c(pStart, runif(nRateCats, minQ, maxQ))
				te<-try(outTries[[10+j]]<-optim(f, par=sp, method="L",  lower=lower, upper=upper), silent=T)
				if(class(te)!="try-error") {
					ltry[10+j]<-outTries[[10+j]]$value
					lsol[10+j,]<-exp(outTries[[10+j]]$par)
				}
				cat(".")

			}
			
			cat("]\n")
			
			ok<-!is.na(ltry)
			
			if(sum(ok)==0) stop("ERROR: No solution found. Does your tree contain zero-length tip branches?")
			
			ltd<-ltry-min(ltry[ok])
			b<-min(which(ltry==min(ltry[ok])))

			gc<-which(ltd<0.1)
			us<-lsol[gc,1]
			usc<-sum((us-min(us))>0.1)			
			b<-min(which(ltry==min(ltry[ok])))
			out<-outTries[[b[1]]]	
			if(usc>1) {out$message="Warning: likelihood surface is flat."}
			
			if(out$convergence!=0) {out$message="Warning: may not have converged to a proper solution."}

			if(out$convergence==0) {out$message="R thinks that this is the right answer."}

			if(treeTransform=="none" & model=="ER") {
				res[[i]]<-list(lnl=-out$value, q=-getQ(exp(out$par), nRateCats, model)[1,1], message=out$message)
			} else if(treeTransform=="none" & model!="ER") {
				res[[i]]<-list(lnl=-out$value, q=getQ(exp(out$par), nRateCats, model), message=out$message)

			} else if(treeTransform=="twoRate") {
				res[[i]]<-list(lnl=-out$value, q=getQ(exp(out$par[-(1:2)]), nRateCats, model), breakpoint=out$par[1], endRate=exp(out$par[2]), message=out$message)
			} else 	res[[i]]<-list(lnl=-out$value, q=getQ(exp(out$par[-1]), nRateCats, model), treeParam=exp(out$par[1]), message=out$message)

				
			if(!is.null(colnames(td$data))) names(res)[i]<-colnames(td$data)[i] else names(res)[i]<-paste("Trait", i, sep="")
		
		if((model=="SYM" | model=="ARD") & nRateCats>2) {
				cat("Plotting surfaces currently not supported for SYM and ARD models unless your character has only two states.\n")
				plotLnlSurf=F
		}
		
		if(plotlnl) {
			cat("Calculating surface\n")
			if(qLimits[1]<=0) {
				cat("Q must be positive, resetting lower plotting limit to 0.00000001")
				qLimits[1]=0.00000001
			}
			if(treeTransform=="none") {
				if(model=="ER") {
					qx<-exp(seq(log(qLimits[1]), log(qLimits[2]), length=50))
					lnl<-numeric(50)
					for(j in 1:50)
						lnl[j]<- -f(log(qx[j]))
					
					lnlDiff<- -out$value-lnl
					plot(qx, lnl, log="x", type="l", xlab="Rate (q)", ylab="lnL")
				} else {
					qx<-exp(seq(log(qLimits[1]), log(qLimits[2]), length=20))
					qy<-exp(seq(log(qLimits[1]), log(qLimits[2]), length=20))
					lnl<-matrix(nrow=20, ncol=20)
					for(j in 1:20)
						for(k in 1:20)
							lnl[j,k]<- -f(log(c(qx[j], qy[k])))
					
					lnlDiff<- -out$value-lnl
					contour(qx, qy, lnl, xlab="Forward Rate", ylab="Backward Rate")
				}	
			} else {
				px<-exp(seq(log(pLimits[1]), log(pLimits[2]), length=20))
				qy<-exp(seq(log(qLimits[1]), log(qLimits[2]), length=20))
				lnl<-matrix(nrow=20, ncol=20)
				for(j in 1:20)
					for(k in 1:20)
						try(lnl[j,k]<- -f(log(c(px[j], qy[k]))))
				
				lnlDiff<- -out$value-lnl
				contour(px, qy, lnlDiff, levels=c(1, 2, 3, 4, 8, 10, 20, 30, 40, 50, 100, 500, 1000), xlab="Tree Transform parameter estimate", ylab="Rate (q)", main="lnL Surface")		
						
				
			}
			
		}
		
	}
	return(res)

}


###Felsenstein's pruning algorithm
### Modified June 13 2008 to work on log-likelihoods instead of likelihoods
likelihoodDiscrete<-function(phy, tip.data, q, model="ER", delta=1, lambda=1,  kappa=1, endRate=1, linear=F, breakPoint=0, f=1, rtt.rescale=0, total.rescale=F, returnFull=F)
{
	
	if(!is.factor(tip.data)) tip.data<-factor(tip.data)
	Q<-getQ(q, n=nlevels(tip.data), model=model)
	if (class(phy) != "phylo")
		stop("object \"phy\" is not of class \"phylo\"");
	#new2old.phylo(phy)->phy ##converts back to old ape tree format with negative values denoting internal nodes
	tmp <- as.numeric(phy$edge)
	nb.tip <- max(tmp) #number of tips
	nb.node <- -min(tmp) #number of internal nodes
	nb.states <- nlevels(tip.data) #numbers of states in character
	l <- matrix(-Inf, nrow=nrow(phy$edge), ncol=nb.states) #makes matrix that will store likelihoods of each character state at each node
	root <- numeric(nb.states) #makes vector that will store root likelihoods
	m <- match(phy$tip.label, names(tip.data)) ##identifies elements of tip.data matrix that corresponds with the tip.label
	if (delta != 1)
		deltaTree(phy, delta) -> phy;
	if (lambda != 1)
		lambdaTree(phy, lambda) -> phy;
	if (kappa != 1)
		kappaTree(phy, kappa) -> phy;
	if (endRate != 1) {
		if(breakPoint!=0) {
			tworateTree(phy, breakPoint, endRate) -> phy;
		} else if(linear==T) {
			linearchangeTree(phy, endRate) -> phy;
		} else exponentialchangeTree(phy, endRate)->phy;
	}


	
	#When comparing deltas across different qs, it might be useful to rescale the total tree length to one	
	if(rtt.rescale!=0)	
		rescaleTree(phy, rtt.rescale) -> phy
		
	new2old.phylo(phy)->phy
	
	for(i in 1:nrow(phy$edge)) #for each edge
		if(as.numeric(phy$edge[i,2])>0) {

			l[i,tip.data[m[as.numeric(phy$edge[i,2])]]] <- 0 
		}
			#if the edge is connected to a terminal taxon, you set the likelihood of the tip value equal to 1 and all others equal to zero.
		times <- branching.times(old2new.phylo(phy)) #get node to tip distances
		-1*(1:(max(as.numeric(names(times)))-min(as.numeric(names(times)))+1))->names(times)
		times = max(times) - times #convert into root to node tips
		if(total.rescale) {
			sum(phy$edge.length) -> total.tree
			phy$edge.length <- phy$edge.length/total.tree
		}
	while(1) {
		
		if(sum(as.numeric(phy$edge[,2])>0)==2) break 
	
		#obtain ancestors of current tips
		x <- match(phy$edge[,1], phy$edge[,2])[as.numeric(phy$edge[,2])>0] #finds nodes connected to terminal taxa
		#find last node with two tip descendent
		a <- max(x[duplicated(x)])
		t <- which(phy$edge[,1]==phy$edge[a,2])
		bl <- phy$edge.length[t]
		age = times[which(names(times)==phy$edge[a,2])]
		l[a,] <- frag.like(l[t,], bl, Q)
		#next line effectively prunes out the tips just used
		phy$edge[a,2]<-1
		phy$edge[t,2]<-0

	
	}
	t <- which(as.numeric(phy$edge[,2])>0)
	bl <- phy$edge.length[t]
	root <- frag.like(l[t,], bl, Q)
	neglnl=-logspace_sum(root)+log(nb.states)
	if(returnFull==F) {
		return(neglnl)
	} else return(list(neglnl=neglnl, root=root, l=l))
}

getQ<-function(q, n, model)
{
	if(model=="ER") Q=evenQ(n)*q
	if(model=="SYM") {
		if(length(q)!=n*(n-1)/2) stop("You must supply the correct number of rate categories.")
		Q<-diag(n)
		xx=1
		for(i in 2:n) {
			for(j in 1:(i-1)) {
				Q[i,j]<-Q[j,i]<-q[xx]
				xx<-xx+1
			}
		}	
		for(i in 1:n) diag(Q)[i]<- -sum(Q[i,-i])
	}
	
	if(model=="ARD") {
		if(length(q)!=n*(n-1)) stop("You must supply the correct number of rate categories.")
		Q<-diag(n)
		xx=1
		for(i in 1:n) {
			for(j in (1:n)[-i]) {
				Q[i,j]<-q[xx]
				xx<-xx+1
			}
		}	
		for(i in 1:n) diag(Q)[i]<- -sum(Q[i,-i])
	}
	
	return(Q)
}

getRateCats<-function(data, model)
{
	if(model=="ER") return(1)
	n<-nlevels(factor(data))
	if(model=="SYM") return(n*(n-1)/2)
	if(model=="ARD") return(n*(n-1))
	
}

##evenQ is an internal function of GEIGER
##This function makes a template for the calculate of a rate matrix that will set all transitions to the same value (based on the total number of states).  One can then multiply the resulting matrix by the overall rate to get the rate matrix for a particular analysis.


`evenQ` <-
function(n)

{

	q<--diag(n)

	q[q==0]<-1/(n-1)

	return(q)

}

#This function calculates the likelihood on one branch of a tree

`frag.like` <-
function(tip.like, bl, q)

{

	nb.states<-ncol(tip.like)

	r<-rep(0, nb.states)

	d<-length(bl)

	p<-list(d)

	for(i in 1:d)

		p[[i]]<-MatrixExp.eig(q*bl[i])

	for(i in 1:nb.states)

		for(j in 1:d) 

			r[i]<-r[i]+logspace_sum(log(p[[j]][i,])+tip.like[j,])

	return(r)

}

#This function is required so that the matrix exponentiation never blows up during the likelihood calculation - but it only works for  symmetric matrices (ie evenQ matrices)


`MatrixExp.simple` <-
function(Q)
{
	n<-nrow(Q)
	res<-matrix(0, nrow=n, ncol=n)
	q<-Q[1,2]
	for(i in 1:n)
		res[i, i]<-1/n+(n-1)/n*exp(-n*q)
	res[res==0]<-1/n-1/n*exp(-n*q)
	return(res)
}

## Code lifted from ape function "ace"
MatrixExp.eig<-
function(Q)
{
	 tmp <- eigen(Q, symmetric = FALSE)
	 P1 <- tmp$vectors %*% diag(exp(tmp$values)) %*% solve(tmp$vectors)
	 return(P1)
	
}

hasZeroLengthTips<-function(phy)
{
	ntips<-length(phy$tip.label)
	tips<-phy$edge[,2]<=ntips
	nn<-phy$edge.length[tips]==0
	if(sum(nn)>0) return(T)
	return(F)
	}
	

logspace_add<-function(logx, logy) {
	if(logx==-Inf) return(logy) else max(logx, logy) + log1p(exp (-abs (logx - logy)));
}

logspace_sum<-function(logx) {
      r<-logx[1]
      if(length(logx)>1)
      	for(i in 2:length(logx))
      		r<-logspace_add(r, logx[i])
      r	
}
`get.simulation.matrix` <-
function(phy)
{
	if (class(phy) != "phylo")
		stop("object \"phy\" is not of class \"phylo\"");
	tmp<-as.numeric(phy$edge)
	nb.tip <- max(tmp)
    bl <- phy$edge.length;
    result<-matrix(0, nrow=nb.tip, ncol=length(bl));

	for(i in 1:nb.tip) {
		j<-i;
		while(j != "-1") {
			branch<-which(as.numeric(phy$edge[,2]) == j);
			result[i, branch]<-sqrt(bl[branch]);
			j<-phy$edge[branch, 1];
		}
	}
	
	return(as.matrix(result));
}


getAncStates<-function(x, phy) {
	
	nn<-phy$Nnode
	nt<-length(phy$tip.label)
	anc<-numeric(nn)
	
	for(i in 1:nn) {
		node<-nt+i
		rp<-root(phy, node=node)
		rp<-multi2di(rp)
		pp<-picAnc(x, rp)
		mm<-match(rp$edge[,1], rp$edge[,2])
		rr<-which(is.na(mm))[1]
		root<-rp$edge[rr,1]
		anc[i]<-pp[root]
		}
		
	names(anc)<-1:nn + nt	
	anc
	
	}



picAnc<-function(x,phy,sd=NULL,n=NULL,se=NULL,scaled=TRUE,var.contrasts=FALSE)
{
	if (class(phy) != "phylo") 
        stop("object 'phy' is not of class \"phylo\"")
    if (is.null(phy$edge.length)) 
        stop("your tree has no branch lengths: you may consider setting them equal to one, or using the function `compute.brlen'.")
    nb.tip <- length(phy$tip.label)
    nb.node <- phy$Nnode
   
    if (length(x) != nb.tip) 
        stop("length of phenotypic and of phylogenetic data do not match")
    if (any(is.na(x))) 
        stop("the present method cannot (yet) be used directly with missing data: you may consider removing the species with missing data from your tree with the function `drop.tip'.")
    phy<-reorder(phy, "pruningwise")

    phenotype<-ssq<-deltaV<-numeric(nb.tip + nb.node)
  	deltaV[1:nb.tip]<-0
    if (is.null(names(x))){
        phenotype[1:nb.tip] <- x
		if(!is.null(n))ssq[1:nb.tip]<-1/n
	}else{
		if (all(names(x) %in% phy$tip.label)){
			phenotype[1:nb.tip] <- x[phy$tip.label]
			if(!is.null(n))ssq[1:nb.tip]<-1/n[phy$tip.label]
		}else{
			phenotype[1:nb.tip] <- x
            if(!is.null(n))ssq[1:nb.tip]<-1/n
            warning("the names of argument \"x\" and the names of the tip labels did not match: the former were ignored in the analysis.")
        }
    }
    contr<-var.contr<-numeric(nb.node)
#pic function brought in from C source
for(i in seq(1,nb.tip*2-2,by=2)){
	j<-i+1
	anc<-phy$edge[i,1]
	d1<-phy$edge[i,2]
	d2<-phy$edge[j,2]
	sumbl<-phy$edge.length[i]+phy$edge.length[j]
	ic<-anc-nb.tip
	contr[ic]<-phenotype[d1]-phenotype[d2]
	if(scaled)contr[ic]<-contr[ic]/sqrt(sumbl)
	if(var.contrasts)var.contr[ic]<-sumbl
	phenotype[anc]<-(phenotype[d1]*phy$edge.length[j]+phenotype[d2]*phy$edge.length[i])/sumbl
	if(j!=(nb.tip*2-2)){
		k<-which(phy$edge[,2]==anc)
		phy$edge.length[k]<-phy$edge.length[k]+phy$edge.length[i]*phy$edge.length[j]/sumbl
	}
	}
	phenotype
}

`ic.sigma` <-
function(phy, data, data.names=NULL)
{
	td<-treedata(phy, data, data.names, sort=T)
	
	f<-function(x) pic(x, td$phy)
	ic<-apply(td$data, 2, f)
	r<-crossprod(ic, ic)/nrow(ic)
	return(r)
}


#takes a phylogeny where the tips of the phylogeny may represent stems of unresolved clades and a list of taxonomic richness for each tip clade and fits a series of birth-death models to each branch in the tree.   Default is estimate d/b. Model limit is the upper limit on splits to be tried. If AIC scores are still improving substanitally after 20 splits, this limit should be increased. 
runMedusa <- function (phy, richness, estimateExtinction=T, modelLimit=20, cutAtStem=T, startR=0.05, startE=0.5, ...) 
{

	if(dim(richness)[2]==2) {
		nr<-dim(richness)[1]
		richness<-data.frame(richness[,1], rep(0, nr), richness[,2])
	}
    phy$node.label <- NULL
    
    #reset model limit if tree has fewer than 20 internal branches
	N<-length(phy$tip.label)
	if(modelLimit>(2*N-2)) modelLimit=2*N-2
   
  #holds parameter estimates  
    allRes<-matrix(nrow=modelLimit+1, ncol=5)
    
    root <- max(phy$edge) - phy$Nnode + 1
    node.list <- 1:max(phy$edge)
    node.list <- node.list[node.list != root]

#First pass estimates a birth-death model that integrates phylogenetic and taxonomic likelihoods described in Rabosky et al., 2007      	  
	  
	res <- list()

	baseModel<-fitDiversification(phy, richness, estimateExtinction=T)

      allRes[1,]<-c(0, baseModel$LH, baseModel$np, baseModel$aic, baseModel$aicc)
      
	#Now split the tree on all branches. 

      for (i in 1:length(node.list)) {
        r1 <- NULL
        r2 <- NULL
        z1 <- NULL
        z2 <- NULL
        z <- NULL
        z <- splitEdgeMatrixGeiger(phy, node.list[i], richness, cutAtStem)
        z1 <- z[z[, 7] == 1, ]
        z2 <- z[z[, 7] == 2, ]
        
        if(length(dim(z1))==0) z1<-rbind(z1)
        if(length(dim(z2))==0) z2<-rbind(z2)

	#Fit BD model to the two subclades of the tree        

        r1 <- getDiversificationModel(z1,  estimateExtinction, startR, startE)
        r2 <- getDiversificationModel(z2, estimateExtinction, startR, startE)

   	 	if(estimateExtinction) np<-5 else np<-3   
    
   
        k<-2*nrow(richness)-1

        res$node[i] <- node.list[i]
        res$LH[i] <- r1$LH + r2$LH
        res$aic[i] <- (-2 * res$LH[i]) + 2*np
        res$aicc[i]<-res$aic[i]+2*np*(np+1)/(k-np-1)

        if(estimateExtinction) {
        	eps1<-r1$par[2]
        	eps2<-r2$par[2] 
 
        } else {
        	eps1=0
            eps2=0
        }
        
        
        res$r1[i] <- r1$par[1]
        res$e1[i] <- eps1
        res$LH1[i] <- r1$LH
        
        res$r2[i] <- r2$par[1]
        res$e2[i] <- eps2
        res$LH2[i] <- r2$LH
        
        res$np[i]<-np
        

      }

      bestModel<-which(res$LH==max(res$LH))[1]
      allRes[2,]<-c(res$node[bestModel], res$LH[bestModel], res$np[bestModel], res$aic[bestModel], res$aicc[bestModel])
      z<-splitEdgeMatrixGeiger(phy, node.list[bestModel], richness, cutAtStem)
      
    
    for(j in 2:modelLimit) {
    	
      res <- list()
      for (i in 1:length(node.list)) {
 
        zNew <- resplitEdgeMatrixGeiger(z, phy, node.list[i], cutAtStem)# resplit edge matrix adds a new split to an already-split tree
        LH=0
        np=0
        for(k in 1:max(zNew[,7])){
        	zPart <- zNew[zNew[, 7] == k, ]
        	if(length(dim(zPart))==0) zPart<-rbind(zPart)
        	if(nrow(zPart)!=0) {
        		rPart <- getDiversificationModel(zPart, estimateExtinction, startR, startE)

        		LH<-LH+rPart$LH
        		if(estimateExtinction) np<-np+3 else np<-np+2
        	}
        	
		}
		np<-np-1 # otherwise you charge one extra for the background
    
   
             k<-2*nrow(richness)-1


        res$node[i] <- node.list[i]
        res$LH[i] <- LH
        res$aic[i] <- (-2 * res$LH[i]) + 2*np
        res$aicc[i]<-res$aic[i]+2*np*(np+1)/(k-np-1)

        res$np[i] <- np

      }

      bestModel<-which(res$LH==max(res$LH))[1]
      allRes[j+1,]<-c(res$node[bestModel], res$LH[bestModel], res$np[bestModel], res$aic[bestModel], res$aicc[bestModel])
      
      z <- resplitEdgeMatrixGeiger(z, phy, node.list[bestModel], cutAtStem)
      cat(j, "\n")
      
    }
    
    return(allRes)
}

# Adds a new split to an already-split edge matrix
resplitEdgeMatrixGeiger <- function (z, phy, node, cutAtStem=T) 
{
	newTag<-max(z[,7])+1
    rootnode <- length(phy$tip.label) + 1

    if (node >= rootnode) {
        node.desc <- node
        pos <- 1
        
        if(cutAtStem) {
        	row1<-which(z[, 2] == node.desc[1])
        	row2<-which(z[, 1] == node.desc[1])
 			row<-c(row1, row2)
 		} else {
 			row<-which(z[, 2] == node.desc[1])
 			}       
        base<-min(z[row,7])
        ok<- z[row,7]==base 
        if(sum(ok)>0)
               z[row[ok],7] <- newTag
           

        while (pos != (length(node.desc) + 1)) {
            temp <- node.sons(phy, node.desc[pos])
            temp <- temp[temp > rootnode]
            if(length(temp)!=0) {
             for (k in 1:length(temp)) {
             	row<-which(z[, 1] == temp[k])
                ok<- z[row,7]==base 
                if(sum(ok)>0)
                	z[row[ok],7] <- newTag
            	}
             node.desc <- c(node.desc, temp)
            }
            pos <- pos + 1
        }
    }
    else if (node > 0) 
        z[z[, 2] == node,7] <- newTag

    return(z)
}


#This returns b and d parameters from composites r (net diversification) and epsilson (death over birth rate)
getBD<-function(r, eps) {
	d<-eps*r/(1-eps)
	b<-r+d
	return(list(b=b, d=d))
	}
	
#This takes a phylogeny and a list of branches where rates shift and returns a list that describes the shift points and new rates.

getFullSplitModel<-function(phy, estimateExtinction=T, breakList, richness, cutAtStem=T){

	res<-list()
	phy$node.label <- NULL
    root <- max(phy$edge) - phy$Nnode + 1
    node.list <- 1:max(phy$edge)
    node.list <- node.list[node.list != root]
    x <- branching.times(phy)

	# First break

    z <- splitEdgeMatrixGeiger(phy, breakList[1], richness, cutAtStem=cutAtStem)
	if(length(breakList)>1)
	  for(i in 2:length(breakList)) {
		z <- resplitEdgeMatrixGeiger(z, phy, breakList[i], cutAtStem=cutAtStem)

		}
	
	 for(k in 1:max(z[,7])){
        	zPart <- z[z[, 7] == k, ]
        	if(length(dim(zPart))==0) zPart<-rbind(zPart)

        	if(nrow(zPart)!=0) {
        		rPart <- getDiversificationModel(zPart, estimateExtinction)
        		res[[k]]<-rPart
        		int<-is.na(zPart[,6])
        		if(sum(int)==0)  {
        			nn<-phy$tip.label[zPart[,2]]
        			
        	        res[[k]]$taxa<-nn

        		} else {
        			tt<-zPart[!int, 2]
        			res[[k]]$taxa<-phy$tip.label[tt]

        			}
        
        	}
        	
		}
		list(res=res, z=z)
}	


fitDiversification <- function (phy, richness, estimateExtinction=T, ...) 
{
    z <- splitEdgeMatrixGeiger(phy, NA, richness, cutAtStem=T)
    r1 <- getDiversificationModel(z, estimateExtinction, ...)
    if(estimateExtinction) np<-2 else np<-1    
    res <- list()
    res$LH <- r1$LH
    res$aic <- (-2 * r1$LH) + 2*np
    n<-nrow(z)
    res$np<-np
    res$k<-n
    res$aicc<-res$aic+2*np*(np+1)/(n-np-1)
    if(estimateExtinction) eps<-r1$par[2] else eps=0
    
    res$r <- r1$par[1]
    res$eps <- eps
    return(res)
}

getDiversificationModel <- function (z, estimateExtinction=T,     startR=0.05, startE=0.5) 
{
    isInt <- is.na(z[,6])
    isTerm <- !isInt
    int<-z[isInt,]
    term<-z[isTerm,]
    
    if(length(dim(int))==0) int<-rbind(int)
    if(length(dim(term))==0) term<-rbind(term)

    nint <- sum(isInt)
    nterm <- sum(isTerm)
    
    lalphaF<-function(b, d, aa, tt) {
    	num<-log(d)+log(exp((b-d)*tt)-1)
    	den<-log(b*(exp((b-d)*tt))-d)     	
    	res<-aa*(num-den)
    	res
    	}	
    
    lbetaF <- function(b, d, aa, tt) {
    	res<-lalphaF(b, d, aa, tt)+log(b)-log(d)
    	res
    }	
    
    lbetaPB <- function(b, aa, tt) {
    	res<-log((exp(b*tt)-1)/(exp(b*tt)))
    }


    Lfunc_comb <- function(r, eps) {
    	if(r<0 | eps<=0 | eps>=1) return(-Inf)

    	bd<-getBD(r, eps)
    	b<-bd$b
    	d<-bd$d
    	
    	
    	if(nint==0) {lint=0} else {
    		branchLengths<-int[,3]-int[,4]
    		lint<-nint * 
            log(r) - r * sum(branchLengths) - sum(log(1 - (eps * 
            exp(-r * int[, 3]))))
        }
    	if(nterm==0) {lterm=0} else {
    		a<-term[,"startR"]
    		n<-term[,"endR"]
    		timeInterval<-term[,"startTime"]-term[,"endTime"]
    		endj<-pmin(a, n)
    		sum<-0
    		lnl<-numeric(nterm)
    		for(i in 1:nterm) {
    			lnxx<-numeric(length=endj[i])
    			for(j in 1:endj[i]) { #using likelihoods from Foote et al. 1999, a correction of Raup
    				logAlpha<-lalphaF(b, d, a[i], timeInterval[i])
    				logBeta<-lbetaF(b, d, a[i], timeInterval[i])
    				if(logBeta>0) logBeta=0
    				s1<-lchoose(a[i],j)+lchoose(n[i]-1,j-1)
    				
    				if(logAlpha==-Inf) s2<-0 else s2<-(a[i]-j)*logAlpha
    				s3<-log(((1-exp(logAlpha))*(1-exp(logBeta)))^j)
    				s4<-(n[i]-1)*logBeta
    				s5<-log(1-exp(logAlpha)) # Conditioning on survival to the present
       				lnxx[j]<-s1+s2+s3+s4-s5

    			}
    			lnl[i]<-logspace_sum(lnxx)
    		}
    		lterm<-sum(lnl)
    	}
    	#return(lterm)

    	return(lint+lterm)
	}
	
	Lfunc_comb_pb <- function(r) {
    	if(r<0) return(-Inf)

    	b<-r
    	d<-0
    	eps<-0
    	
    	if(nint==0) {lint=0} else {
    		branchLengths<-int[,3]-int[,4]
    		lint<-nint * 
            log(r) - r * sum(branchLengths) - sum(log(1 - (eps * 
            exp(-r * int[, 3]))))
        }
    	if(nterm==0) {lterm=0} else {
    		a<-term[,"startR"]
    		n<-term[,"endR"]
    		timeInterval<-term[,"startTime"]-term[,"endTime"]
    		endj<-pmin(a, n)
    		sum<-0
    		lnl<-numeric(nterm)
    		for(i in 1:nterm) {
    			lnxx<-numeric(length=endj[i])
    			for(j in 1:endj[i]) { #using likelihoods from Foote et al. 1999, a correction of Raup
    				logAlpha<- -Inf
    				logBeta<-lbetaPB(b, a[i], timeInterval[i])
    				s1<-lchoose(a[i],j)+lchoose(n[i]-1,j-1)
    				
    				if(logAlpha==-Inf) s2<-0 else s2<-(a[i]-j)*logAlpha
    				s3<-log(((1-exp(logAlpha))*(1-exp(logBeta)))^j)
    				s4<-(n[i]-1)*logBeta
    				s5<-log(1-exp(logAlpha)) # Conditioning on survival to the present
       				lnxx[j]<-s1+s2+s3+s4-s5

    			}
    			lnl[i]<-logspace_sum(lnxx)
    		}
    		lterm<-sum(lnl)
    	}
    	
    	return(lint+lterm)
	}

	
    res <- list()
    

    
    if (estimateExtinction == TRUE) {
     	foo <- function(x) 
     		- Lfunc_comb(r=exp(x[1]), eps=exp(x[2]))
		sp<-log(c(startR, startE))
		o<-optim(foo, par=sp, method="N")
		res$LH <- -o$value
    	res$par <- exp(o$par)

    } else {
    	foo <- function(x) 
     		- Lfunc_comb_pb(r=exp(x[1]))
		sp<-log(startR)
		o<-optimize(foo, interval=c(-100, 1))
		res$LH <- -o$objective
    	res$par <- exp(o$minimum)
    	}

	    



    return(res)
}


plotDiversificationSurface <- function (phy, nPoints=10, rInterval=c(0.00001, 0.3), eInterval=NULL, logTransform=T) 
{
    z <- splitEdgeMatrixGeiger(phy, phy$Nnode)
	rootnode = (length(phy$tip.label) + 1)
 	int <- z[z[, 2] > rootnode, ]
    term <- z[z[, 2] < rootnode, ]
    nint <- nrow(int)
    nterm <- nrow(term)
    betaF <- function(r, eps, t1) {
    	if(r<0 | eps<0 | eps>=1) return(-Inf)
        xf <- (exp(r * t1) - 1)/(exp(r * t1) - eps)
        xf
    }

    Lfunc_comb <- function(r, eps) {
    	if(r<0 | eps<0 | eps>=1) return(-Inf)

        (sum(log(1 - betaF(r, eps, term[1:nterm, 4]))) + sum((term[1:nterm, 
            5] - 1) * log(betaF(r, eps, term[1:nterm, 4]))) + nint * 
            log(r) - r * sum(int[1:nint, 4]) - sum(log(1 - (eps * 
            exp(-r * int[1:nint, 3])))))
    }
	
	if(logTransform) {
			x<-exp(seq(from=log(rInterval[1]), to=log(rInterval[2]), length.out=nPoints))
	} else x<-seq(from=rInterval[1], to=rInterval[2], length.out=nPoints)
	
	if(is.null(eInterval)) {
		lnl<-numeric(nPoints)
		for(i in 1:nPoints)
			lnl[i]<-Lfunc_comb(r=x[i], eps=0)
		plot(x, lnl, type="l")	
	} else {
		if(logTransform) {
			y<-exp(seq(from=log(eInterval[1]), to=log(eInterval[2]), length.out=nPoints))
		} else y<-seq(from=eInterval[1], to=eInterval[2], length.out=nPoints)
		lnl<-matrix(nrow=nPoints, ncol=nPoints)
		for(i in 1:nPoints)
			for(j in 1:nPoints)
				lnl[i,j]<-Lfunc_comb(r=x[i], eps=y[j])
		mm<-max(lnl)		
		contour(lnl, levels=c(mm-1, mm-2, mm-3, mm-4, mm-5, mm-10, mm-100), labels=c(1, 2, 3, 4, 5, 10, 100), axes=F, xlab="b-d", ylab="d/b")
		tics<-floor(c(1, nPoints/4, nPoints/2, nPoints*3/4, nPoints))
		axis(1, at=c(0, 0.25, 0.5, 0.75, 1), labels=round(x[tics], 3))
		axis(2, at=c(0, 0.25, 0.5, 0.75, 1), labels=round(y[tics], 3))
	}
}




fitSplitModel <- function (phy, estimateExtinction=T,     startR=0.05, startE=0.5) 
{
    phy$node.label <- NULL
    root <- max(phy$edge) - phy$Nnode + 1
    node.list <- 1:max(phy$edge)
    node.list <- node.list[node.list != root]
    x <- branching.times(phy)
    res <- list()
    for (i in 1:length(node.list)) {
        r1 <- NULL
        r2 <- NULL
        z1 <- NULL
        z2 <- NULL
        z <- NULL
        z <- splitEdgeMatrixGeiger(phy, node.list[i])
        z1 <- z[z[, 6] == 1, ]
        z2 <- z[z[, 6] == 2, ]
        
        r1 <- getDiversificationModel(z1, estimateExtinction, startR, startE)
        r2 <- getDiversificationModel(z2, estimateExtinction, startR, startE)

   	 	if(estimateExtinction) np<-4 else np<-2    
    
   
        

        res$node[i] <- node.list[i]
        res$LH[i] <- r1$LH + r2$LH
        res$aic[i] <- (-2 * res$LH[i]) + 2*np*2
        
        if(estimateExtinction) {
        	eps1<-r1$par[2]
        	eps2<-r2$par[2] 
 
        } else {
        	eps1=0
            eps2=0
        }
        
        
        res$r1[i] <- r1$par[1]
        res$e1[i] <- eps1
        res$LH1[i] <- r1$LH
        
        res$r2[i] <- r2$par[1]
        res$e2[i] <- eps2
        res$LH2[i] <- r2$LH

    }
    res <- as.data.frame(res)
    return(res)
}




splitEdgeMatrixGeiger <- function (phy, node, richness, cutAtStem=T) 
{
    bt <- branching.times(phy)
    rootnode <- length(phy$tip.label) + 1
    
    #m1<-match(richness[,1], phy$tip.label)
    #m2<-match(phy$edge[,1], names(bt))

    interior<-phy$edge[,2] %in% phy$edge[,1]
 	tips<-!interior 
 	
 	z<-matrix(ncol=7, nrow=sum(interior)+nrow(richness))
    colnames(z)<-c("anc", "dec", "startTime", "endTime", "startR", "endR", "Group")
  
 	for(i in 1:sum(interior)) {
 		anc<-phy$edge[interior,1][i]
 		dec<-phy$edge[interior,2][i]
 		startTime<-bt[names(bt)==anc]
 		endTime<-startTime-phy$edge.length[interior][i]
 		sr<-1
 		er<-NA
 		group<-NA
 		z[i,]<-c(anc, dec, startTime, endTime, sr, er, group)
 		} 
  
  	m1<-match(richness[,1], phy$tip.label)

  
	for(i in 1:nrow(richness)) {
		wt<-phy$tip.label[m1][i]
		we<-which(phy$edge[,2]==m1[i])
		anc<-phy$edge[we,1]
 		dec<-phy$edge[we,2]
 		startTime<-bt[names(bt)==anc]
 		endTime<-richness[i,2]
 		sr <- 1
 		er<-richness[i,3]
 		group<-NA
 		z[i+sum(interior),]<-c(anc, dec, startTime, endTime, sr, er, group)
		}
	 
	for(i in 1:max(z[,2])) {
		nn<-which(z[,2]==i)
		o<-order(z[nn,4], decreasing=T)
		if(length(nn)>1) for(j in 2:length(nn)) {
			z[nn,][o[j],3]<-z[nn,][o[j-1],4]
			z[nn,][o[j],5]<-z[nn,][o[j-1],6]
			}
		}
	if(is.na(node)) {
		z[,7]<-1
	} else {
		setZ<-function(z, n) {
			if(n %in% z[,2]) {
       			ok<-z[,2]==n
       			z[ok,7]<-2
				ns<-node.sons(phy, n)
				if(length(ns)!=0) for(k in 1:length(ns))
					z=setZ(z, ns[k])
			} else if(n %in% z[,1]) z[,7]<-2
			z
		}
		
		z<-setZ(z, node)
        
    	z[is.na(z[,7]),7]<-1
    }            
    
    if(!cutAtStem) {
    	row<-which(z[, 1] == node)
    	if(length(row)>0)
			z[row,7]<-1
    	}
       
    return(z)
}
##########################



summaryMedusa<-function(phy, richness, out, cutoff=4, plotTree=T, useCorrection=F, cutAtStem=T) {
	breaks<-numeric()
	i=2
	if(useCorrection) col=5 else col=4
	while(1) {
		if((out[i-1,col]-out[i,col])<cutoff) break;
		breaks[i-1]<-out[i,1]
		i<-i+1
		}
	rr<-getFullSplitModel(phy, breakList=breaks, richness=richness, cutAtStem=cutAtStem)
	
	if(plotTree) {
		mm<-match(phy$edge[,2], rr$z[,2])
		plot(phy, edge.color=rr$z[mm,7])
		}
	
	
	rr$res	
	}

`name.check` <-
function(phy, data, data.names=NULL)
{
	if(is.null(data.names)) 
	{
		if(is.vector(data))
			data.names=names(data)
		else
			data.names<-rownames(data)
	}
	t<-phy$tip.label
	r1<-t[is.na(match(t,data.names))]
	r2<-data.names[is.na(match(data.names,t))]
	
	r<-list(sort(r1), sort(r2))
	
	names(r)<-cbind("Tree.not.data", "Data.not.tree")
	if(length(r1)==0 && length(r2)==0) return("OK")
	else return(r)
}


`node.leaves` <-
function(phy, node)
{
	node<-as.numeric(node)
	n<-length(phy$tip.label);
	if(node <= n) return(phy$tip.label[as.numeric(node)])

	l<-character();
	
	d<-node.sons(phy, node);
	for(j in d) {
		if(j <= n) l<-c(l, phy$tip.label[as.numeric(j)])
		else l<-c(l, node.leaves(phy, j));
	}
	return(l);
}

'node.sons' <-
function(phy, node)
{
	r<-which(phy$edge[,1]==node)
	return(phy$edge[r,2])	
}
phy.anova<-function(phy, data, group, data.names=NULL, nsim=1000)
{
	td<-treedata(phy, data, data.names)
	
	s<-mean(pic(td$data, td$phy)^2)
	a<-anova(lm(td$data~group))
	f.data<-a[1,4]
	sims<-sim.char(td$phy, as.matrix(s), nsim=nsim)
	
	foo<-function(xx) anova(lm(xx~group))[1,4]
	f.null<-apply(sims, 3, foo)

	cat("Standard ANOVA:\n")	
	print(a)

	cat("\n\nPhylogenetic p-value: \t")
	cat((sum(f.null>f.data)+1)/(nsim+1))

		
}

phy.manova<-function(phy, data, group, data.names=NULL, nsim=1000, test="Wilks")
{	
	td<-treedata(phy, data, data.names)

	s<-ic.sigma(td$phy, td$data)
	
	m<-summary.manova(manova(as.matrix(td$data)~group), test=test)
	
	w.data<-m[[4]][1,2]
	
	sims<-sim.char(td$phy, s, nsim=nsim)
	
	foo<-function(xx) summary.manova(manova(as.matrix(xx)~group), test=test)[[4]][1,2]
	
	w.null<-apply(sims, 3, foo)

	cat("Standard MANOVA:\n")	
	print(m)
	
	cat("\n\nPhylogenetic p-value: \t")
	cat((sum(w.data>w.null)+1)/(nsim+1))
	
}
prune.extinct.taxa<-function (phy, tol = .Machine$double.eps^0.5) 
{
    if (class(phy) != "phylo") 
        stop("object \"phy\" is not of class \"phylo\"")
    phy2 <- phy
    phy <- new2old.phylo(phy)
    tmp <- as.numeric(phy$edge)
    nb.tip <- max(tmp)
    nb.node <- -min(tmp)
    xx <- as.numeric(rep(NA, nb.tip + nb.node))
    names(xx) <- as.character(c(-(1:nb.node), 1:nb.tip))
    xx["-1"] <- 0
    for (i in 2:length(xx)) {
        nod <- names(xx[i])
        ind <- which(as.numeric(phy$edge[, 2]) == nod)
        base <- phy$edge[ind, 1]
        xx[i] <- xx[base] + phy$edge.length[ind]
    }
    depth <- max(xx)
    offset <- depth - xx[names(xx) > 0]
    drops <- phy$tip.label[offset > tol]
    if (length(drops) >= (nb.tip - 1)) 
        return(NULL)
    if(length(drops)==0)
    	return(phy2)    
    res <- drop.tip(phy2, drops)
    res
}

prune.random.taxa<-function (phy, n) 
{
    if (class(phy) != "phylo") 
        stop("object \"phy\" is not of class \"phylo\"")
    nb.tip <- length(phy$tip.label)
    if (n > nb.tip) 
        return(NULL)
    cut <- sample(1:nb.tip, n)
    r <- drop.tip(phy, cut)
    return(r)
}
`rate.estimate` <-
function(time=0, n=0, phy=NULL, epsilon = 0, missing = 0, crown=TRUE, kendall.moran=FALSE)
{
	if(!is.null(phy)) {
		if (class(phy) != "phylo") 
       		stop("object \"phy\" is not of class \"phylo\"")
       	
		if(kendall.moran)
		{
			if(missing != 0)
				cat("WARNING: current implementation of Kendall-Moran estimate does not account for missing taxa\n")
			return(kendallmoran.rate(phy))
		}
    	time<-max(branching.times(phy))
		n<-length(phy$tip.label)
	
		n<-n+missing;
		crown=TRUE;
	}
	
    if(crown==TRUE) {
    	if(epsilon==0) {
			rate=(log(n)-log(2))/time
    	} else {
 			rate=1/time*(log(n/2*(1-epsilon^2)+
 					2*epsilon+1/2*(1-epsilon)*sqrt(n*(n*epsilon^2-
 					8*epsilon+2*n*epsilon+n)))-log(2))
    	}
    
    } else {
    	if(epsilon==0) {
			rate=log(n)/time
    	} else {
 			rate=1/time*log(n*(1-epsilon)+epsilon)
    	}
    
    
    }

   return(rate)
}

'kendallmoran.rate' <-
function(phy)
{
	s<-sum(phy$edge.length)
	rate<-(length(phy$tip.label)-2)/s
	return(rate)
}

'crown.p' <- 
function(time, r, epsilon, n)
{
b<-((exp((r*time)))-1)/((exp((r*time)))-epsilon)
a<-epsilon*b
p<-(((b^(n-2)))*((n*(1-a-b+(a*b)))+a+(2*b)-1))/(1+a)
return(p)
}

'stem.p' <-
function(time, r, epsilon, n)
{
b<-((exp((r*time)))-1)/((exp((r*time)))-epsilon)
p<-(b^(n-1))
return(p)
}

'stem.limits' <-
function(r, epsilon, time, prob=c(0.025, 0.975))
{
	limits <- matrix(nrow=length(time), ncol=2)
	for (i in 1:length(time)) {
		beta <- (exp(r*time[i])-1)/(exp(r*time[i]) - epsilon) #From M&S '01 2b
		alpha <- epsilon * beta #From M&S '01 2a
		u <- (log(beta) + log(prob[1]))/log(beta) #From M&S '01 10a
		l <- (log(beta) + log(prob[2]))/log(beta)
		limits[i, 1] <- l
		limits[i, 2] <- u
	}
	return(limits)
}

'crown.limits' <- 
function(r, epsilon, time, prob=c(0.025, 0.975))
{
	limits <- matrix(nrow=length(time), ncol=2)
	for (i in 1:length(time))
	{
		foo<-function(x, prob)
			(crown.p(time[i], r, epsilon, x)-prob)^2
		u<-optim(foo, par=exp(r*time), prob=prob[1], method="L-BFGS-B")
		l<-optim(foo, par=exp(r*time), prob=prob[2], method="L-BFGS-B")

		limits[i, 1] <- l$par
		limits[i, 2] <- u$par
	}
	return(limits)
}


`rc` <-
function(phy, make.plot=TRUE, plot.bonf=FALSE, p.cutoff=0.05, cex = par("cex"))
{

	
	nb.tip <- length(phy$tip.label)
	nb.node <- phy$Nnode
    
	nd<-branching.times(phy);
	node.depth<-c(nd, rep(0, nb.tip))
	names(node.depth)<-c(names(nd), as.character(1:nb.tip))
	p<-numeric(nb.node)
	max.desc<-numeric(nb.node)
	num.anc<-numeric(nb.node)
	node.name<-character(nb.node)

	stem.depth<-numeric();
	stem.depth[1]<-node.depth[1];
	
	for(i in 2:length(node.depth)) {
			anc<-which(phy$edge[,2]==names(node.depth)[i])
			stem.depth[i]<-node.depth[names(node.depth)==phy$edge[anc,1]]
	}
	

		
	ltt<-sort(nd, decreasing=TRUE)
	
	for(i in 2:length(ltt)) {
		nn<-stem.depth>=ltt[i-1]&node.depth<ltt[i-1]
		anc<-sum(nn)
		desc<-numeric(anc)
		pp<-numeric(anc)
		num.anc[i]<-anc
		for(j in 1:anc) {
			desc[j]<-length(node.leaves(phy, as.numeric(names(nn)[nn][j])))
		}
		max.desc[i]<-max(desc)
		p[i]<-rcp(max.desc[i], nb.tip, anc)
		node.name[i]<-names(ltt[i])
	}
	num.anc[1]<-1
	max.desc[1]<-nb.tip
	p[1]<-1
	bonf.p<-pmin(p*length(ltt),1)
	res<-cbind(num.anc, max.desc, p, bonf.p)
	rownames(res)<-c("root", node.name[2:nb.node])
	
	if(make.plot) {
		labels<-character(length(phy$edge.length))
		names(labels)<-as.character(1:length(labels)+nb.tip)
		if(plot.bonf) {
			s<-which(res[,4]<p.cutoff)
		} else s<-which(res[,3]<p.cutoff)
		mark<-character(length(s))
		if(length(s)>0) {
		for(i in 1:length(s)) {
			xx<-names(s)[i]
			tt<-which(ltt==ltt[xx])
			nn<-stem.depth>=ltt[tt-1]&node.depth<ltt[tt-1]
			anc<-sum(nn)
			desc<-numeric(anc)
			for(j in 1:anc) {
				desc[j]<-length(node.leaves(phy, names(nn)[nn][j]))
			}
			bigone<-which(desc==max(desc))
			mark[i]<-names(nn)[nn][bigone]
		}
		}
		labels[mark]<-"*"
		phy$node.label<-labels
		plot.phylo(phy, show.node.label=T, cex=cex, no.margin=T)
	}
	return(res)
}

`rcp` <-
function(ni, n, k) 
{
	max<-floor((n-k)/(ni-1))
	sum=0
	for(v in 0:max) {
		term=(-1)^v*choose(k, v)*choose(n-v*(ni-1)-1, k-1)
		sum=sum+term
 	}
	return(1-(sum/choose(n-1, k-1)))
}



`set.seed.clock` <-
function(print=F){
	date = date()
 	seed1 = as.numeric(strsplit(substring(date,12,19),":")[[1]])%*%c(1,100,10000)
 	seed <- runif(1, min=0, max=50) * seed1
 	set.seed(seed)
 	if(print) cat("Seed = ", seed, "\n");
 	seed[1,1]
}


`sim.char` <-
function(phy, model.matrix, nsims=1, model="brownian", root.state=1)
{
	phy<-new2old.phylo(phy)
	
	nchar<-nrow(model.matrix);
	m<-get.simulation.matrix(phy);
	nbranches<-ncol(m);	
	nspecies<-nrow(m);

	if(model=="brownian" | model=="speciational")
	{
		if(model=="speciational") {
			m[m>0]<-1.0;
		}
	
		rnd<-t(mvrnorm(nsims*nbranches, mu=rep(0, nchar), Sigma=model.matrix));
		rnd<-array(rnd, dim=c(nchar, nbranches, nsims));
		
		simulate<-function(v) m %*% as.matrix(v);
		
		result<-apply(rnd, 1, simulate)
		result<-aperm(array(result, dim=c(nspecies, nsims, nchar)), c(1, 3, 2))
		
		rownames(result)<-phy$tip.label;
	}
	
	if(model=="discrete")
	{
		nchar<-length(model.matrix);
		node.value<-numeric(nbranches)
		result<-array(0, dim=c(nspecies, nchar, nsims))
		for(j in 1:nchar) {
			m<-model.matrix[[j]];
			for(k in 1:nsims) {
	   			for(i in 1:nbranches) {
					if(as.numeric(phy$edge[i,1])==-1) s<-root.state
					else {
						parent<-which(phy$edge[,2]==phy$edge[i,1])
						s<-node.value[parent]
					}	
					p<-MatrixExp(m*phy$edge.length[i])
					probs<-cumsum(p[s,])
					r<-runif(1)
					node.value[i]<-min(which(r<probs))
	   			}
	   			result[,j,k]<-node.value[as.numeric(phy$edge[,2])>0]
	   		}	
		}
		rownames(result)<-phy$tip.label;

	}		
	return(result);
}


`tip.disparity` <-
function(phy, data, data.names=NULL, disp="avg.sq")
{
	if (class(phy) != "phylo")
		stop("object \"phy\" is not of class \"phylo\"");
	
	td<-treedata(phy, data, data.names)

		
	nb.tip <- length(td$phy$tip.label)
    nb.node <- td$phy$Nnode
    result<-numeric();
    
    for(i in 1:nb.node) {
    	l<-node.leaves(td$phy, nb.tip+i);
    	d<-td$data[match(l, row.names(data)),];
    	result[i]<-disp.calc(d, disp);
    }
    return(result);	  	
}


# Treedata is a function internal to GEIGER
# that makes sure that the names of the taxa 
# in the tree and data file match, and prunes 
# things accordingly; it returns a list with 
# two elements: phy and data

# data.names is optional, and will replace the names or rownames
# of data when matching data to the tree

# if sort is T, data will have rows in the same order
# as the taxon names in phy$tip.label

treedata<-function(phy, data, data.names=NULL, sort=F, warnings=T)
{

	if(is.vector(data)) data<-as.matrix(data)
	if(is.factor(data)) data<-as.matrix(data)
	if(is.array(data) & length(dim(data))==1) data<-as.matrix(data)

	if(is.null(data.names)) {
		if(is.null(rownames(data))) {
				data.names<-phy$tip.label[1:dim(data)[1]]
				if(warnings)
					cat("Warning: no tip labels, order assumed to be the same as in the tree\n")
			} else
				data.names<-rownames(data)
	}
	nc<-name.check(phy, data, data.names)
	if(is.na(nc[[1]][1]) | nc[[1]][1]!="OK") {
		if(length(nc[[1]]!=0)) {
			phy=drop.tip(phy, as.character(nc[[1]]))
			if(warnings) {
				cat("Dropped tips from the tree because there were no matching names in the data:\n")
				print(nc[[1]])
				cat("\n")
			}
		}
	
		if(length(nc[[2]]!=0)) {
			m<-match(data.names, nc[[2]])
			data=as.matrix(data[is.na(m),])
			data.names<-data.names[is.na(m)]
			if(warnings) {
				cat("Dropped rows from the data because there were no matching tips in the tree:\n")
				print(nc[[2]])
				cat("\n")
			}
		}
 	}
	order<-match(data.names, phy$tip.label)	

	rownames(data)<-phy$tip.label[order]
	
	if(sort) {

    	index <- match(phy$tip.label, rownames(data))
   		data <- as.matrix(data[index,])
	}
	
	phy$node.label=NULL
	
	return(list(phy=phy, data=data))
}
